/*!******************************************************************/
/*  File: gesort.c                                                  */
/*  =============                                                   */
/*                                                                  */
/*  This file includes:                                             */
/*                                                                  */
/*  GEsort_1() Sort array and remove dupletts                       */
/*  GEsort_2() Sort array-1 with respect to array-2                 */
/*                                                                  */
/*  This file is part of the VARKON Geometry Library.               */
/*  URL:  http://www.varkon.com                                     */
/*                                                                  */
/*  This library is free software; you can redistribute it and/or   */
/*  modify it under the terms of the GNU Library General Public     */
/*  License as published by the Free Software Foundation; either    */
/*  version 2 of the License, or (at your option) any later         */
/*  version.                                                        */
/*                                                                  */
/*  This library is distributed in the hope that it will be         */
/*  useful, but WITHOUT ANY WARRANTY; without even the implied      */
/*  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR         */
/*  PURPOSE.  See the GNU Library General Public License for more   */
/*  details.                                                        */
/*                                                                  */
/*  You should have received a copy of the GNU Library General      */
/*  Public License along with this library; if not, write to the    */
/*  Free Software Foundation, Inc., 675 Mass Ave, Cambridge,        */
/*  MA 02139, USA.                                                  */
/*                                                                  */
/*  (C)Microform AB 1984-1999, Gunnar Liden, gunnar@microform.se    */
/*                                                                  */
/********************************************************************/

#include "../../DB/include/DB.h"
#include "../include/GE.h"

/*!******************************************************************/

        DBstatus GEsort_1(
        DBfloat *uin,
        short    nin,
        short   *nut,
        DBfloat *uut)

/*      Sort array of float numbers and remove values
 *      equal within +/- TOL4.
 *
 *      In:   uin  = Unsorted values
 *            nin  = Number of unsorted values
 *
 *      Out: *nut  = Number of sorted values
 *           *uut  = Sorted values
 *
 *      (C)microform ab 8/3/95 J. Kjellander
 *
 *      1996-06-03 Bug vid > 2 lika lsn, J.Kjellander
 *      1999-05-18 Rewritten, J.Kjellander
 *
 ******************************************************************!*/

 {
   short   i,start,end,imin,j;
   bool    shift;
   DBfloat min,tmp,diff;

/*
***If uin <> uut copy uin to uut.
*/
   if ( uut != uin ) for ( i=0; i<nin; ++i ) *(uut+i) = *(uin+i);

  *nut = nin;
/*
***Sen sorterar vi. Bubble-sort width decreasing depth.
*/
   start = 0;
   end   = nin - 1;

loop:
   shift =  FALSE;
   min   = *(uut+start);
   imin  =  start;

   for ( i=start; i<end; ++i )
     {
     if ( *(uut+i+1) < *(uut+i) )
       {
       shift = TRUE;
       tmp = *(uut+i); *(uut+i) = *(uut+i+1); *(uut+i+1) = tmp;
       if ( *(uut+i) < min )
         {
         min = *(uut+i);
         imin = i;
         }
       }
     }

   if ( shift )
     {
     tmp = *(uut+imin); *(uut+imin) = *(uut+start); *(uut+start) = tmp;
     start++;
     end--;
     goto loop;
     }
/*
***Remove all dupletts.
*/
   end = nin;

   for ( i=0; i<end-1; ++i )
     {
     diff = *(uut+i) - *(uut+i+1);
     if ( ABS(diff) < TOL4 )
       {
       for ( j=i; j<end-1; ++j ) *(uut+j) = *(uut+j+1);
       end--;
       i--;   /* Bugfix, 1996-06-03 JK */
       }
     }

   *nut = (short)end;

   return(0);
 }

/********************************************************************/
/********************************************************************/

      DBstatus GEsort_2(
      DBfloat uin1[],
      DBfloat uin2[],
      short   noin,
      short  *pnout,
      DBfloat uout1[],
      DBfloat uout2[])

/*    Orders array-2 with respect to array-1 and removes 
 *    values equal within +/- TOL4.
 *
 *    In: uin1  = The first input array
 *        uin2  = The second input array
 *        noin  = The number of elements in uin1 (and uin2)
 *
 *    Out: *pnout = The number of output elements
 *         *uout1 = Ordered array 1
 *         *uout2 = Ordered array 2
 *
 *     (C)Microform AB,  Gunnar Liden 1985-02-03
 *
 *     1985-07-10 Error system  B. Doverud
 *     1999-05-18 Rewritten, J.Kjellander
 *
 ********************************************************************/

 {
   DBfloat cuin1[GMMXSG*INTMAX]; /* A copy of uin1 in order not to destroy */
                                 /* the input vector                       */
   DBfloat cuin2[GMMXSG*INTMAX]; /* A copy of uin2 in order not to destroy */
                                 /* the input vector                       */
   short    k,i ;                /* Loop index                             */
   short   nout;                 /* The number of output points (=*pnout)  */
   short   imin=0;               /* For the ordering of the elements       */

/*
***Copy uin1 to cuin1 and uin2 to cuin2 in order not to
***destroy the input data
*/
   for ( k=0; k<noin; k++ )
     {
     cuin1[k] = uin1[k];
     cuin2[k] = uin2[k];
     }
/*
***Start value for nout is noin
*/
   nout = noin;
/*
***Sort the elements in cuin1 to uout1 and let uin2 be sorted
***the same way into uout2
*/
   for ( k=1; k <= noin; k++ )
     { 
     uout1[k-1] = TOL6;
     if ( k > 1 ) cuin1[imin-1] = 1.1*TOL6;

     for ( i=1; i <= noin; i++ )
      {
/*
***Check if elements are equal
*/
      if ( ABS(cuin1[i-1] - uout1[k-1]) < TOL4 )
        {
      --nout;
        cuin1[i-1] = 1.1*TOL6;
        }
      if ( cuin1[i-1] <= uout1[k-1] )
        {
        uout1[k-1] = cuin1[i-1];
        uout2[k-1] = cuin2[i-1];
        imin = i;
        }
      }
    }
/*
***The end.
*/
  *pnout = nout;

   return(0);
 }

/********************************************************************/
